﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.Patterns.Core
{
    public class PersonService2
    {
        public PersonService2(
            IPersonValidator validator,
            IDuplicateEmailValidator duplicateEmailValidator,
            IRepository<IPerson> repository)
        {
            if (validator == null)
                throw new ArgumentNullException("validator", "validator is null.");
            if (duplicateEmailValidator == null)
                throw new ArgumentNullException("duplicateEmailValidator", "duplicateEmailValidator is null.");
            if (repository == null)
                throw new ArgumentNullException("repository", "repository is null.");

            _PersonValidator = validator;
            _DuplicateEmailValidator = duplicateEmailValidator;
            _PersonRepository = repository;
        }

        private IPersonValidator _PersonValidator;
        private IDuplicateEmailValidator _DuplicateEmailValidator;
        private IRepository<IPerson> _PersonRepository;

        public void Save(IPerson saveThis)
        {
            if (_PersonValidator.IsValid(saveThis) == false)
            {
                throw new InvalidOperationException("Cannot save an invalid person.");
            }

            if (saveThis.Id == 0 && 
                _DuplicateEmailValidator.IsEmailInUse(saveThis.EmailAddress) == true)
            {
                throw new InvalidOperationException("Cannot save a person with a duplicate email address.");
            }

            try
            {
                _PersonRepository.Save(saveThis);
            }
            catch (SqlServerOutOfDiskSpaceException ex)
            {
                new Logger().WriteToLog(ex.ToString());

                throw;
            }
        }
    }
}