using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.Patterns.Core
{
    public class PersonService3
    {
        public PersonService3(
            IPersonValidator validator,
            IDuplicateEmailValidator duplicateEmailValidator,
            IRepository<IPerson> repository,
            ILogger logger)
        {
            if (logger == null)
                throw new ArgumentNullException("logger", "logger is null.");
            if (validator == null)
                throw new ArgumentNullException("validator", "validator is null.");
            if (duplicateEmailValidator == null)
                throw new ArgumentNullException("duplicateEmailValidator", "duplicateEmailValidator is null.");
            if (repository == null)
                throw new ArgumentNullException("repository", "repository is null.");

            _PersonValidator = validator;
            _DuplicateEmailValidator = duplicateEmailValidator;
            _PersonRepository = repository;
            _Logger = logger;
        }

        private IPersonValidator _PersonValidator;
        private IDuplicateEmailValidator _DuplicateEmailValidator;
        private IRepository<IPerson> _PersonRepository;
        private ILogger _Logger;

        public void Save(IPerson saveThis)
        {
            if (_PersonValidator.IsValid(saveThis) == false)
            {
                throw new InvalidOperationException("Cannot save an invalid person.");
            }

            if (saveThis.Id == 0 &&
                _DuplicateEmailValidator.IsEmailInUse(saveThis.EmailAddress) == true)
            {
                throw new InvalidOperationException("Cannot save a person with a duplicate email address.");
            }

            try
            {
                _PersonRepository.Save(saveThis);
            }
            catch (SqlServerOutOfDiskSpaceException ex)
            {
                // new Logger().WriteToLog(ex.ToString());
                _Logger.WriteToLog(ex.ToString());

                throw;
            }
        }
    }
}
