﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Patterns.Core;
using Benday.Patterns.Core.Fakes;

namespace Benday.Patterns.UnitTests
{
    [TestClass]
    public class PersonServiceFixture2
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            _PersonValidatorInstance = null;
            _DuplicateEmailValidatorInstance = null;
            _RepositoryInstance = null;
        }
        
        private PersonService2 _SystemUnderTest;
        public PersonService2 SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new PersonService2(
                        PersonValidatorInstance,
                        DuplicateEmailValidatorInstance,
                        RepositoryInstance);
                }

                return _SystemUnderTest;
            }
        }

        private StubIPersonValidator _PersonValidatorInstance;
        public StubIPersonValidator PersonValidatorInstance
        {
            get
            {
                if (_PersonValidatorInstance == null)
                {
                    _PersonValidatorInstance = new StubIPersonValidator();
                }

                return _PersonValidatorInstance;
            }
        }

        private StubIDuplicateEmailValidator _DuplicateEmailValidatorInstance;
        public StubIDuplicateEmailValidator DuplicateEmailValidatorInstance
        {
            get
            {
                if (_DuplicateEmailValidatorInstance == null)
                {
                    _DuplicateEmailValidatorInstance = new StubIDuplicateEmailValidator();
                }

                return _DuplicateEmailValidatorInstance;
            }
        }

        private InMemoryRepository<IPerson> _RepositoryInstance;
        public InMemoryRepository<IPerson> RepositoryInstance
        {
            get
            {
                if (_RepositoryInstance == null)
                {
                    _RepositoryInstance = new InMemoryRepository<IPerson>();
                }

                return _RepositoryInstance;
            }
        }

        private void IsValidPersonShouldReturn(bool returnValue)
        {
            PersonValidatorInstance.IsValidIPerson = 
                (x) => { return returnValue; };   
        }

        private void IsEmailInUseShouldReturn(bool returnValue)
        {
            DuplicateEmailValidatorInstance.IsEmailInUseString = 
                (x) => { return returnValue; };
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SaveInvalidPersonThrowsException()
        {
            // arrange
            IsValidPersonShouldReturn(false);

            // act
            SystemUnderTest.Save(new StubIPerson());

            // assert
            // (the assert is the [ExpectedException])
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void SaveValidPersonWithDuplicateEmailThrowsException()
        {
            // arrange
            IsValidPersonShouldReturn(true);
            IsEmailInUseShouldReturn(true);

            // act
            SystemUnderTest.Save(new StubIPerson());

            // assert
            // (the assert is the [ExpectedException])
        }

        [TestMethod]
        public void SaveValidPersonWithNonDuplicateSavesAndGetsNonZeroIdValue()
        {
            // arrange
            IsValidPersonShouldReturn(true);
            IsEmailInUseShouldReturn(false);

            IPerson person = new StubIPerson();

            // act
            SystemUnderTest.Save(person);

            // assert
            Assert.AreNotEqual<int>(0, person.Id, "Id should not be zero.");
        }
    }
}
