﻿using Benday.Patterns.Core;
using Benday.Patterns.Core.Fakes;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.Patterns.UnitTests
{
    [TestClass]
    public class PersonServiceFixture3
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
            _PersonValidatorInstance = null;
            _DuplicateEmailValidatorInstance = null;
            _RepositoryInstance = null;
            _LoggerInstance = null;
        }

        private PersonService3 _SystemUnderTest;
        public PersonService3 SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new PersonService3(
                        PersonValidatorInstance,
                        DuplicateEmailValidatorInstance,
                        RepositoryInstance,
                        LoggerInstance);
                }

                return _SystemUnderTest;
            }
        }

        private StubILogger _LoggerInstance;
        public StubILogger LoggerInstance
        {
            get
            {
                if (_LoggerInstance == null)
                {
                    _LoggerInstance = new StubILogger();
                }

                return _LoggerInstance;
            }
        }

        private StubIPersonValidator _PersonValidatorInstance;
        public StubIPersonValidator PersonValidatorInstance
        {
            get
            {
                if (_PersonValidatorInstance == null)
                {
                    _PersonValidatorInstance = new StubIPersonValidator();
                }

                return _PersonValidatorInstance;
            }
        }

        private StubIDuplicateEmailValidator _DuplicateEmailValidatorInstance;
        public StubIDuplicateEmailValidator DuplicateEmailValidatorInstance
        {
            get
            {
                if (_DuplicateEmailValidatorInstance == null)
                {
                    _DuplicateEmailValidatorInstance = new StubIDuplicateEmailValidator();
                }

                return _DuplicateEmailValidatorInstance;
            }
        }

        private StubIRepository<IPerson> _RepositoryInstance;
        public StubIRepository<IPerson> RepositoryInstance
        {
            get
            {
                if (_RepositoryInstance == null)
                {
                    _RepositoryInstance = new StubIRepository<IPerson>();
                }

                return _RepositoryInstance;
            }
        }

        private void IsValidPersonShouldReturn(bool returnValue)
        {
            PersonValidatorInstance.IsValidIPerson =
                (x) => { return returnValue; };
        }

        private void IsEmailInUseShouldReturn(bool returnValue)
        {
            DuplicateEmailValidatorInstance.IsEmailInUseString =
                (x) => { return returnValue; };
        }

        [TestMethod]
        [ExpectedException(typeof(SqlServerOutOfDiskSpaceException))]
        public void WhenSqlServerOutOfDiskSpaceExceptionHappensTheLoggerIsCalled()
        {
            // arrange
            IsValidPersonShouldReturn(true);
            IsEmailInUseShouldReturn(false);

            RepositoryInstance.SaveT0 +=
                (x) => {
                    throw new SqlServerOutOfDiskSpaceException();
                };

            bool wasWriteToLogCalled = false;

            LoggerInstance.WriteToLogString +=
                (x) => {
                    wasWriteToLogCalled = true;
                };

            // act
            SystemUnderTest.Save(new StubIPerson());

            // assert
            Assert.IsTrue(wasWriteToLogCalled, "WriteToLog() wasn't called.");
        }
    }
}
