﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Benday.Patterns.Core;
using Benday.Patterns.Core.Fakes;

namespace Benday.Patterns.UnitTests
{
    [TestClass]
    public class PersonValidatorFixture
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;       
        }

        private PersonValidator _SystemUnderTest;
        public PersonValidator SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new PersonValidator();
                }

                return _SystemUnderTest;
            }
        }


        private IPerson GetPersonAndPopulate(string firstName, 
            string lastName, string emailAddress)
        {
            IPerson person = new StubIPerson();

            person.FirstName = firstName;
            person.LastName = lastName;
            person.EmailAddress = emailAddress;

            return person;
        }

        [TestMethod]
        public void WhenAllFieldsAreNullThePersonIsInvalid()
        {
            IPerson person =
                GetPersonAndPopulate(null, null, null);

            Assert.IsFalse(
                SystemUnderTest.IsValid(person));
        }

        [TestMethod]
        public void WhenFirstNameIsNullThePersonIsInvalid()
        {
            IPerson person =
                GetPersonAndPopulate(null, "test_ln", "info@benday.com");

            Assert.IsFalse(
                SystemUnderTest.IsValid(person));
        }

        [TestMethod]
        public void WhenFirstNameIsEmptyStringThePersonIsInvalid()
        {
            IPerson person =
                GetPersonAndPopulate(String.Empty, "test_ln", "info@benday.com");

            Assert.IsFalse(
                SystemUnderTest.IsValid(person));
        }

        [TestMethod]
        public void WhenLastNameIsNullThePersonIsInvalid()
        {
            IPerson person =
                GetPersonAndPopulate("test_fn", null, "info@benday.com");

            Assert.IsFalse(
                SystemUnderTest.IsValid(person));
        }

        [TestMethod]
        public void WhenLastNameIsEmptyStringThePersonIsInvalid()
        {
            IPerson person =
                GetPersonAndPopulate("test_fn", String.Empty, "info@benday.com");

            Assert.IsFalse(
                SystemUnderTest.IsValid(person));
        }

        [TestMethod]
        public void WhenEmailAddressIsNullThePersonIsInvalid()
        {
            IPerson person =
                GetPersonAndPopulate("test_fn", "test_ln", null);

            Assert.IsFalse(
                SystemUnderTest.IsValid(person));
        }

        [TestMethod]
        public void WhenEmailAddressIsEmptyStringThePersonIsInvalid()
        {
            IPerson person =
                GetPersonAndPopulate("test_fn", "test_ln", String.Empty);

            Assert.IsFalse(
                SystemUnderTest.IsValid(person));
        }

        [TestMethod]
        public void WhenAllFieldsArePopulatedThePersonIsValid()
        {
            IPerson person =
                GetPersonAndPopulate("test_fn", "test_ln", "info@benday.com");

            Assert.IsTrue(
                SystemUnderTest.IsValid(person));
        }
    }
}