﻿using Benday.SampleApp2.Web;
using Benday.SampleApp2.Web.Controllers;
using Benday.SampleApp2.Web.Core;
using Benday.SampleApp2.Web.Models;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace Benday.SampleApp2.Tests
{
    [TestClass]
    public class PresidentAdapterFixture
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
        }


        private PresidentAdapter _SystemUnderTest;
        public PresidentAdapter SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new PresidentAdapter();
                }

                return _SystemUnderTest;
            }
        }

        private XElement GetPresidentAsXElement()
        {
            XDocument doc = XDocument.Parse(WebDataResource.UsPresidents);

            return doc.Root.ElementByLocalName("president");
        }

        [TestMethod]
        public void GetPresidentAsXElementReturnsNonNull()
        {
            var fromValue = GetPresidentAsXElement();

            Assert.IsNotNull(fromValue);
        }



        [TestMethod]
        public void AdaptPresidentFromXElement()
        {
            var fromValue = GetPresidentAsXElement();

            President toValue = new President();

            SystemUnderTest.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        private void AssertAreEqual(XElement expected, President actual)
        {
            string firstName = expected.AttributeValue("firstname");
            string lastName = expected.AttributeValue("lastname");
            int id = expected.AttributeValueAsInt32("id");
            int startYear = expected.AttributeValueAsInt32("start");
            int endYear = expected.AttributeValueAsInt32("end");
            string filename = expected.AttributeValue("image-filename");

            Assert.AreEqual<string>(firstName, actual.FirstName, "FirstName");
            Assert.AreEqual<string>(lastName, actual.LastName, "LastName");
            Assert.AreEqual<int>(id, actual.Id, "Id");
            Assert.AreEqual<int>(startYear, actual.StartYear, "StartYear");
            Assert.AreEqual<int>(endYear, actual.EndYear, "EndYear");
            Assert.AreEqual<string>(filename, actual.ImageFilename, "ImageFilename");
        }
    }
}
