using Benday.SampleApp2.Web;
using Benday.SampleApp2.Web.Controllers;
using Benday.SampleApp2.Web.Core;
using Benday.SampleApp2.Web.Models;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web.Mvc;
using System.Xml.Linq;

namespace Benday.SampleApp2.Tests
{
    [TestClass]
    public class PresidentControllerFixture
    {
        [TestInitialize]
        public void OnTestInitialize()
        {
            _SystemUnderTest = null;
        }


        private PresidentController _SystemUnderTest;
        public PresidentController SystemUnderTest
        {
            get
            {
                if (_SystemUnderTest == null)
                {
                    _SystemUnderTest = new PresidentController();
                }

                return _SystemUnderTest;
            }
        }

        private T AssertActionResultAndGetModel<T>(ActionResult actionResult) where T : class
        {
            Assert.IsNotNull(actionResult, "Action result was null.");
            Assert.IsInstanceOfType(actionResult, typeof(ViewResult),
                "Action result was the wrong type.");

            var result = actionResult as ViewResult;

            Assert.IsNotNull(result.Model, "Model was null.");
            Assert.IsInstanceOfType(result.Model, typeof(T), "Model was the wrong data type.");

            var model = result.Model as T;

            return model;
        }

        [TestMethod]
        public void WhenFirstCalledTheModelIsNotNullAndIsExpectedType()
        {
            AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Index());
        }

        [TestMethod]
        public void WhenFirstCalledTheSearchFieldsAreBlank()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Index());

            Assert.AreEqual<string>(String.Empty,
                model.SearchByFirstName,
                "SearchByFirstName should be empty.");

            Assert.AreEqual<string>(String.Empty,
                model.SearchByLastName,
                "SearchByLastName should be empty.");
        }

        [TestMethod]
        public void WhenFirstCalledTheResultsAreNonNull()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Index());

            Assert.IsNotNull(model.Results, "Results collection was null.");
        }

        [TestMethod]
        public void WhenFirstCalledTheResultsAreZeroLength()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Index());

            Assert.AreEqual<int>(0, model.Results.Count, "Result count was wrong.");
        }

        [TestMethod]
        public void SearchByEmptyFirstNameAndLastNameReturnsAllPresidents()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Search(String.Empty, String.Empty));

            Assert.AreEqual<int>(44, model.Results.Count, "Wrong result count.");
        }

        [TestMethod]
        public void SearchByFirstNameForValidPresidentReturnsResults_Barack()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Search("barack", String.Empty));

            Assert.AreEqual<int>(1, model.Results.Count, "Result count should not be 0.");
        }

        [TestMethod]
        public void SearchByFirstNameForValidPresidentReturnsResults_John()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Search("john", String.Empty));

            Assert.AreNotEqual<int>(0, model.Results.Count, "Result count should not be 0.");
            Assert.AreEqual<int>(4, model.Results.Count, "Result count was wrong.");
        }

        [TestMethod]
        public void SearchByLastNameForValidPresidentReturnsResults()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Search(String.Empty, "jefferson"));

            Assert.AreEqual<int>(1, model.Results.Count, "Result count should not be 0.");
        }

        [TestMethod]
        public void SearchByFirstNameAndLastNameForValidPresidentReturnsResults()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Search("rich", "nixon"));

            Assert.AreEqual<int>(1, model.Results.Count, "Result count should not be 0.");
        }

        [TestMethod]
        public void SearchByFirstNameAndLastNameForInvalidPresidentReturnsNoResults()
        {
            var model = AssertActionResultAndGetModel<PresidentSearch>(
                SystemUnderTest.Search("tauphin", "mauphinburphy"));

            Assert.AreEqual<int>(0, model.Results.Count, "Result count should not be 0.");
        }
    }
}
