﻿'use strict';

describe('theApp', function () {
    var scope;
    var controller;
    beforeEach(function () {
        module('app.controllers');
    });

    describe('PresidentCtrl with real service', function () {
        beforeEach(inject(function ($rootScope, $controller, _presidentRepository_) {
            scope = $rootScope.$new();
            controller = $controller('PresidentCtrl', {
                '$scope': scope, presidentService: _presidentRepository_
            });
        }));

        it('should have an instance of repository', function () {
            expect(scope.repositoryInstance).toBeDefined();
        });

        it('should not tell you the time', function () {

            expect(scope.theTime).toEqual("i'm not telling you.");
        });

        it('should have a blank first name at the start', function () {
            expect(scope.firstName).toEqual('');
        });

        it('should have a blank last name at the start', function () {
            expect(scope.lastName).toEqual('');
        });

        it('should have a zero record count at the start', function () {
            expect(scope.recordCount).toEqual(0);
        });

        it('should reset fields to blank when reset is called', function () {
            scope.firstName = 'zach';
            scope.lastName = 'taylor';

            scope.reset();

            expect(scope.firstName).toEqual('');
            expect(scope.lastName).toEqual('');
        });

        /*
        it('get back one record for zach taylor search', function () {
            scope.firstName = 'zach';
            scope.lastName = 'taylor';

            runs(function () {
                scope.search();
            });
            
            waitsFor(function () {
                return scope.recordCount > 0;
            }, "never got back any records", 5000)

            runs(function () {
                expect(scope.recordCount).toEqual(1);
            });
            
        });

        it('get back no record for tauphin mauphinburphy search', function () {
            scope.firstName = 'tauphin';
            scope.lastName = 'mauphinburphy';

            scope.search();

            expect(scope.recordCount).toEqual(0);
        });
        */
    });

    describe('PresidentCtrl with mock service', function () {
        var repositoryInstance;

        beforeEach(inject(function ($rootScope, $controller) {

            repositoryInstance = jasmine.createSpyObj('presidentRepository', ['getCurrentTime', 'runSearch']);
            repositoryInstance.getCurrentTime.and.returnValue('yay! i love mittens!');

            scope = $rootScope.$new();
            controller = $controller('PresidentCtrl', {
                '$scope': scope, presidentRepository: repositoryInstance
            });
        }));

        it('should have an instance of repository', function () {
            expect(scope.repositoryInstance).toBeDefined();
        });

        it('should say "i love mittens"', function () {

            //just assert. $scope was set up in beforeEach() (above)
            expect(scope.theTime).toEqual('yay! i love mittens!');
        });

        it('should get back 3 records and record count should be 3', function () {
            repositoryInstance.runSearch.and.callFake(function (first, last, callback) {
                var data = {
                    Results: ['row0', 'row1', 'row2']
                };

                callback(data);
            });

            scope.search();

            expect(scope.recordCount).toEqual(3);
        });

        it('should get back 0 records and record count should be 0', function () {
            repositoryInstance.runSearch.and.callFake(function (first, last, callback) {
                var data = {
                    Results: []
                };

                callback(data);
            });

            scope.search();

            expect(scope.recordCount).toEqual(0);
        });
    });
});


